#include "gd.h"

// #define SERIAL_DEBUG

// coordinate offsets (relative to centre position) for drawing the ring.  It consists of 88 pixels.
const int8_t ringXoffset[88] PROGMEM = { 
  -2,-1,0,1,2,
  -4,-3,-2,-1,0,1,2,3,4,
  -5,-4,-3,-2,-1,1,2,3,4,5,
  -5,-4,-3,3,4,5,
  -6,-5,-4,4,5,6,
  -6,-5,-4,4,5,6,
  -6,-5,5,6,
  -6,-5,-4,4,5,6,
  -6,-5,-4,4,5,6,
  -5,-4,-3,3,4,5,
  -5,-4,-3,-2,-1,1,2,3,4,5,
  -4,-3,-2,-1,0,1,2,3,4,
  -2,-1,0,1,2  
};

const int8_t ringYoffset[88] PROGMEM = {
  -6,-6,-6,-6,-6,
  -5,-5,-5,-5,-5,-5,-5,-5,-5,
  -4,-4,-4,-4,-4,-4,-4,-4,-4,-4,
  -3,-3,-3,-3,-3,-3,
  -2,-2,-2,-2,-2,-2,
  -1,-1,-1,-1,-1,-1,
  0,0,0,0,
  1,1,1,1,1,1,
  2,2,2,2,2,2,
  3,3,3,3,3,3,
  4,4,4,4,4,4,4,4,4,4,
  5,5,5,5,5,5,5,5,5,
  6,6,6,6,6
};

int8_t xd = 2, yd = 2; // amount ring moves per step.  Initially to the right and down.
uint8_t ringOldX = 6, ringOldY = 6; // first (previous) ring centre coordinates
uint8_t ringNewX = ringOldX + xd, ringNewY = ringOldY + yd; // next ring center coordinates.

void drawRing(uint8_t x, uint8_t y) { // draw ring centred on coordinates (x, y): just flip (XOR) the 88 pixels that comprise the ring
  for (uint8_t i = 0; i < 88; i++) {
    GDgWritePixel(x + pgm_read_byte_near(ringXoffset + i), y + pgm_read_byte_near(ringYoffset + i), 2);
  }
}

void setup() {
#ifdef SERIAL_DEBUG
  Serial.begin(115200);
  Serial.println("Started");
#endif
  
  GDinit();
  GDtWrite("Hello, World!", 0, 1);
  GDtWrite("LCD module", 1, 3);
  GDtWrite("12864B V2.0", 2, 2);
  GDtWrite("Parallel intface", 3, 0);
  GDgDisplay(true); // turn graphics on
  GDflipBanks(); // show the text
  GDtCopyBank(); // copy the text so it's in both banks
  drawRing(ringOldX, ringOldY); // draw ring in 1st position to off-screen buffer
  GDflipBanks(); // reveal the ring
  drawRing(ringNewX, ringNewY); // draw ring in next position to the off-screen buffer
}

void loop() {
  static int delayMillis = 400; // start ring movement off slowly, with a 400ms per animation frame delay (in addition to the drawing time)
  GDflipBanks(); // reveal the newly drawn ring
  drawRing(ringOldX, ringOldY); // redraw ring in previous position to the off-screen buffer: because flip (xor) mode is used this erases the previously drawn ring
  // could just use GDgClear(); instead, but with the 88 pixel ring, the redraw-to-erase is faster.  With more complex graphics, use GDgClear();
  ringOldX = ringNewX, ringOldY = ringNewY;
  ringNewX += xd; ringNewY += yd; // move ring
  drawRing(ringNewX, ringNewY); // draw ring in new position to the off-screen buffer
  if (ringNewX <= 7 || ringNewX >= 119) xd = -xd;
  if (ringNewY <= 7 || ringNewY >= 55) yd = -yd; // bounce at screen edges
  if (delayMillis) {
    delay(delayMillis);
    delayMillis -= 4; // gradually accelerate till delayMillis is zero and animation speed is limited by how fast the code can erase and redraw the 88 pixels
    // it quickly gets so fast it becomes almost invisible (the liquid crystal display is too slow to reveal the animation)
    // if (delayMillis < 16) delayMillis = 16; // enable this line to limit speed to something more visible
  }
  // GDgDisplay(millis() % 10000 < 5000); // alternate five seconds of showing the graphics with five seconds of hiding them.  Any text always remains visible
}
