// support 'library' for 12864B V2.0 graphic LCD panel
// done in C rather than C++ to migrate to STC (8051) chips using SDCC
// ceptimus 2022-05-29

// for Nano/Uno, etc. connections are:
// LCD   Nano
//  D0     2
//  D1     3
//  D2     4
//  D3     5
//  D4     6
//  D5     7
//  D6     8
//  D7     9
//  RS    10 PORTB bit 2
//  RW    11 PORTB bit 3
//  E     12 PORTB bit 4
//  PSB   not needed: can be connected to VCC. Was originally connected to 13 PORTB bit 5
//  RST   not needed: can be connected to VCC. Was originally connected to A0 PORTC bit 0

// No need to connect anything to LCD contrast pin 3 - with default jumper settings, the working contrast is on the back of the display
// I connected the LED backlight +ve (pin 19) to Arduino 5V VCC via 470R current limiting resistor

#ifndef GD_H
#define GD_H

#include <Arduino.h>

#define GD_RS_HIGH  asm("sbi 0x05, 2 \n")
#define GD_RS_LOW   asm("cbi 0x05, 2 \n")
#define GD_RW_HIGH  asm("sbi 0x05, 3 \n")
#define GD_RW_LOW   asm("cbi 0x05, 3 \n")
#define GD_E_HIGH   asm("sbi 0x05, 4 \n")
#define GD_E_LOW    asm("cbi 0x05, 4 \n")
// #define GD_PSB_HIGH asm("sbi 0x05, 5 \n")
// #define GD_PSB_LOW  asm("cbi 0x05, 5 \n")
// #define GD_RST_HIGH asm("sbi 0x08, 0 \n")
// #define GD_RST_LOW  asm("cbi 0x08, 0 \n")

void GDinit(void);

// display has enough RAM (character RAM and graphic RAM) to hold exactly 2 'screens'.
// could be used for vertical smooth-scrolling (horizontal scrolling a problem due to screen layout)
// but here used for a double-buffered display.  Usually write to the 'off screen' display, then flip to showing that.
// normally just clear, draw, then call GDflipBanks() to display - don't worry about which banks are currently used.
// if you don't want double buffering, just call GDdoubleBuffering(false), then there's no need to use GDflipBanks
// call GDdoubleBuffering(true) to switch it back on.
void GDflipBanks(void);
void GDdoubleBuffering(bool on); 
// text mode fuctions:  Display is 4 rows of 16 ASCII chars. Row 0 overflows to row 2; row 1 to row 3; row 2 to row 1. Any attempted Row 3 overflow is clipped
void GDtClear(void); // clears GDcurrentBank
void GDtWrite(uint8_t *text, uint8_t row, uint8_t col); // row 0 - 3, col 0 - 15
void GDtCopyBank(void); // copies entire text 'screen' from GDdisplayedBank to GDcurrentBank

// graphic mode functions: 128x64 pixels
void GDgDisplay(bool graphicsOn); // turn graphics display on or off
void GDgClear(void); // clear GDcurrentBank graphics
void GDgWritePixel(uint8_t x, uint8_t y, uint8_t mode); // mode: 0=clear pixel, 1=set pixel, 2=flip existing pixel to opposite state
bool GDgReadPixel(uint8_t x, uint8_t y);

#endif
